"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceProxyFactory = void 0;
require("reflect-metadata");
const consts_1 = require("./consts");
class ServiceProxyFactory {
    constructor(broker) {
        this.broker = broker;
    }
    create(t) {
        const serviceName = Reflect.getMetadata(consts_1.METADATA_KEY_NAME, t.prototype);
        const actionsMetadata = Reflect.getMetadata(consts_1.METADATA_KEY_ACTIONS, t.prototype);
        const eventsMetadata = Reflect.getMetadata(consts_1.METADATA_KEY_EVENTS, t.prototype);
        if (!serviceName) {
            throw new Error('Provided class is not decorated as "proxyableService"');
        }
        const handler = {
            get: (target, prop) => {
                const action = actionsMetadata && actionsMetadata[prop];
                const event = eventsMetadata && eventsMetadata[prop];
                if (!action && !event) {
                    throw new Error(`Can't find action ${prop}`);
                }
                const paramsNames = Reflect.getMetadata(`${consts_1.METADATA_KEY_ACTIONS}:${prop}:params`, t.prototype);
                return (...params) => {
                    let param;
                    if (!paramsNames) {
                        param = params[0];
                    }
                    else {
                        param = {};
                        paramsNames.forEach((name, i) => (param[name] = params[i]));
                    }
                    if (action) {
                        return this.broker.call(`${serviceName}.${action.name}`, param);
                    }
                    else {
                        return this.broker.emit(`${serviceName}.${event.name}`, param);
                    }
                };
            },
        };
        return new Proxy({}, handler);
    }
}
exports.ServiceProxyFactory = ServiceProxyFactory;
//# sourceMappingURL=proxyFactory.js.map